package ms.warpzone.warpzonestatuswidget;

import java.util.Random;

import android.annotation.TargetApi;
import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Build;
import android.os.StrictMode;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.widget.RemoteViews;

public class WidgetProvider extends AppWidgetProvider {

   @Override
   public void onUpdate(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
      Log.d("onUpdate", "called");
      
      // get the widget views
      RemoteViews remoteViews = new RemoteViews(context.getPackageName(), R.layout.widget_layout);
      
      // loading animation
      remoteViews.setImageViewResource(R.id.imageView, android.R.drawable.stat_notify_sync);//ic_popup_sync);
      for (int widgetId : appWidgetIds)
          sendUpdate(context, appWidgetManager, widgetId, remoteViews);
      
      // load the settings
	  SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
	  
	  // keep the refresh timer running
	  updateAlarm(context, settings, appWidgetIds);
	  
	  // load status and send notification, if appropriate
      int status = StatusLoader.getStatusImageID();
      if(statusChanged(context, settings, status) && settings.getBoolean("notifications_status_change", false))
    	  sendNotification(context, settings);
      
      // set new status image
      remoteViews.setImageViewResource(R.id.imageView, status);
      for (int widgetId : appWidgetIds)
         sendUpdate(context, appWidgetManager, widgetId, remoteViews);
   }
   
   private void updateAlarm(Context context, SharedPreferences settings, int[] appWidgetIds) {
      Intent intent = get_ACTION_APPWIDGET_UPDATE_Intent(context);
      PendingIntent pendingIntent = PendingIntent.getBroadcast(context, appWidgetIds[0], intent, PendingIntent.FLAG_UPDATE_CURRENT);
      
      AlarmManager alarm = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
      alarm.cancel(pendingIntent);
      int minutes = Integer.parseInt(settings.getString("sync_frequency", "15"));
      if(minutes > 0)
      {
    	  long interval = 1000*60*minutes;
    	  alarm.setRepeating(AlarmManager.ELAPSED_REALTIME_WAKEUP, SystemClock.elapsedRealtime() + interval, interval, pendingIntent);
      }
   }
   
   private void sendNotification(Context context, SharedPreferences settings) {

	  int icon = settings.getInt("status", -1);
	  Uri sound = Uri.parse(settings.getString("notifications_status_change_ringtone", "content://settings/system/notification_sound"));
	  boolean vibrate = settings.getBoolean("notifications_status_change_vibrate", false);
	  
	  String title = "Warpzone Status";
	  String message = "Der Warpzone-Status hat sich geändert!";
	   
	  Intent notificationIntent = new Intent(context, WidgetProvider.class);
	  PendingIntent contentIntent = PendingIntent.getActivity(context, 0, notificationIntent, 0);
	   
	  Notification notification = new Notification(icon, message, System.currentTimeMillis());
	  notification.sound = sound;
	  notification.flags = Notification.FLAG_AUTO_CANCEL;
	  if(vibrate)
		  notification.defaults = Notification.DEFAULT_VIBRATE;
	  notification.setLatestEventInfo(context, title, message, contentIntent);
	  
	  NotificationManager nm = (NotificationManager) context.getSystemService(Context.NOTIFICATION_SERVICE);
	  nm.notify(new Random().nextInt(), notification);
   }
   
   private boolean statusChanged(Context context, SharedPreferences settings, int status_new)
   {
	  if(status_new != R.drawable.warpzone_logo_small && settings.getInt("status", -1) != status_new)
	  {
		  SharedPreferences.Editor editor = settings.edit();
		  editor.putInt("status", status_new);
		  editor.commit();
		  return true;
	  }
	  return false;
   }
   
   private void sendUpdate(Context context, AppWidgetManager appWidgetManager, int appWidgetId, RemoteViews remoteViews)
   {
      //When user click on the label, update ALL the instances of the widget.
      Intent labelIntent = get_ACTION_APPWIDGET_UPDATE_Intent(context);
      PendingIntent labelPendingIntent = PendingIntent.getBroadcast(context, appWidgetId, labelIntent, PendingIntent.FLAG_UPDATE_CURRENT);
      remoteViews.setOnClickPendingIntent(R.id.imageView, labelPendingIntent);
      Log.d("updateAppWidget", "Updated ID: " + appWidgetId);
      //Call the Manager to ensure the changes take effect.
      appWidgetManager.updateAppWidget(appWidgetId, remoteViews);
   }
   
   /**
    * Utility method to ensure that when we want an Intent that fire ACTION_APPWIDGET_UPDATE, the extras are correct.<br>
    * The default implementation of onReceive() will discard it if we don't add the ids of all the instances.
    * @param context
    * @return
    */
   protected Intent get_ACTION_APPWIDGET_UPDATE_Intent(Context context) {
      AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
      ComponentName thisAppWidget = new ComponentName(context.getPackageName(), WidgetProvider.class.getName());
      int[] appWidgetIds = appWidgetManager.getAppWidgetIds(thisAppWidget);
      Intent intent = new Intent(context, WidgetProvider.class);
      intent.setAction(AppWidgetManager.ACTION_APPWIDGET_UPDATE);
      intent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS, appWidgetIds);
      return intent;
   }
}
